#!/usr/bin/env python

"""
gui_router.py
Copyright (C) 2006  Yannick Le Teigner

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

import gtk
import base64
import zlib
import nvram_io
from dynamips_lib import *


dynamips = [] # list of dynamips servers

class GuiRouter(object):
    def __init__(self, x, y):
        self.started = False
        self.suspended = False

        self.px = self.x = x
        self.py = self.y = y
        self.dx = self.dy = 0

        self.connections = []

        self.colorbg_r, self.colorbg_g, self.colorbg_b = 0.4, 0.6, 0.6

        self.w = self.h = 10

        self.name_delta = 0

        self.push_cfg = False
        # plain text config, as when initially pushed to the router
        self.saved_config = None
        self.tmp_config64bz = None
        # config retrieved before stopping the device
        self.stopped_config64bz = None


    def inside(self, x, y):
        """ returns true if x, y is inside the device """
        return x>self.x and x<self.x+self.w and y>self.y and y<self.y+self.h


    def moveto(self, x, y):
        """ move the device to position x, y, preserving all positions for drawing """
        tx = x - self.x - self.dx
        ty = y - self.y - self.dy
        for l,l,c,s in self.connections:
            if s == 0:
                c.translate0to(tx, ty)
            else:
                c.translate1to(tx, ty)
        self.px = self.x #old position
        self.py = self.y
        self.x = x - self.dx
        self.y = y - self.dy

    def setdelta(self, x, y):
        """ the x,y delta to which the pointer clicked inside the device """
        self.dx = x - self.x
        self.dy = y - self.y


    def init_draw(self, context):
        #px = max(context.device_to_user_distance(1, 1))
        #fascent, fdescent, fheight, fxadvance, fyadvance = \
        #        context.font_extents()
        xbearing, ybearing, width, height, xadvance, yadvance = \
                context.text_extents(self.name)
        self.name_delta = width/2


    def set_bgcolor(self, r, g, b):
        self.colorbg_r, self.colorbg_g, self.colorbg_b = r, g, b

    def _get_gdkcolor(self):
        return gtk.gdk.Color(int(self.colorbg_r*65535),
                             int(self.colorbg_g*65535),
                             int(self.colorbg_b*65535))

    gdkcolor = property(_get_gdkcolor, doc='return the gtk.gdk background color')

    def inside_empty_port(self, x, y):
        """ returns true if x, y is inside one of the device's empty PA's port """
        slot_nb, xl, yl = self.which_pa(x, y)
        slot_pa = self.slot[slot_nb]
        if slot_pa is not None:
            port = slot_pa.which_port(xl, yl)
            for lpa,lpo,c,s in self.connections:
                if slot_nb==lpa and port==lpo:
                    return False
        return True

    def addconnection(self, connection, side):
        if side==0:
            # this pa
            pa_slot, xl, yl = self.which_pa(connection.xa, connection.ya)
            local_pa = self.slot[pa_slot] #error
            local_port = local_pa.which_port(xl, yl)
            # remote pa
            remote_device = connection.deviceb
            if isinstance(remote_device, GuiRouter): # if remote is a router
                pa_slotb, xlb, ylb = remote_device.which_pa(
                                            connection.xb, connection.yb)
                remote_pa = remote_device.slot[pa_slotb]
                remote_port = remote_pa.which_port(xlb, ylb)
                # connect the ports
                #def connect(self, localint, localport, remoteserver, remoteadapter, remoteint, remoteport = None):
                local_pa.connect(local_pa.technology, local_port, remote_device.dynamips,
                                 remote_pa, remote_pa.technology, remote_port)
            elif isinstance(remote_device, FRSW): # if it is a FR switch
                port = remote_device.which_port(connection.xb, connection.yb)
                remote_device.connect(port, self.dynamips, local_pa, local_port)
                remote_device.set_port(port, [connection, 1 ^ side])
            elif isinstance(remote_device, ATMSW): # if it is an ATM switch
                port = remote_device.which_port(connection.xb, connection.yb)
                remote_device.connect(port, self.dynamips, local_pa, local_port)
                remote_device.ports[port] = [connection, 1 ^ side]
            elif isinstance(remote_device, ETHSW): # a ethernet switch
                port = remote_device.which_port(connection.xb, connection.yb)
                remote_device.connect(port, remote_device.dynamips,
                                      local_pa, local_port)
            else:
                print self, remote_device, "no connect"
        elif side==1:
            # this pa
            pa_slot, xl, yl = self.which_pa(connection.xb, connection.yb)
            local_pa = self.slot[pa_slot]
            local_port = local_pa.which_port(xl, yl)
        self.connections.append([pa_slot, local_port, connection, side])

    def delete_connection(self, connection):
        """Delete the connection given as a parameter from this router"""
        for cctl in self.connections:
            pa_slot, local_port, cct, side = cctl
            # deleting the connection
            if connection == cct:
                self.slot[pa_slot].delete_nio(local_port)
                self.connections.remove(cctl)
                return

    def has_config(self):
        return self.saved_config is not None

    def __get_config_gz64(self):
        config_b64 = self.config_b64
        config = base64.b64decode(config_b64)
        config_gz = zlib.compress(config)
        config_gzb64 = base64.b64encode(config_gz)
        return config_gzb64

    def __set_config_gz64(self, config64bz):
        self.saved_config = nvram_io.read_conf_b64gz(config64bz)
        self.config_b64 = base64.b64encode(self.saved_config)

    config_gz64 = property(__get_config_gz64, __set_config_gz64, doc = "The gzip compressed, base 64 encoded configuration of the router")

    def set_tmp_config_gz64(self, config64bz):
        self.tmp_config64bz = config64bz
