#!/usr/bin/env python

"""
util.py
Copyright (C) 2006, 2007  Yannick Le Teigner

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

import gtk
from dynamips_lib import Router

__all__ = ['setup_ghost', 'RouterSelectionDialog']
ghosts = []

def setup_ghost(device):
    ghost_instance = device.imagename + '-' + device.dynamips.host
    ghost_file = device.imagename + '.ghost'

    if not ghost_instance in ghosts:
        ghost = Router(device.dynamips, device.model, 'ghost-'+ghost_instance,
                       consoleFlag = False)
        ghost.image = device.image
        # set the maxram
        ghost.ram = device.ram
        ghost.ghost_status = 1
        ghost.ghost_file = ghost_file
        ghost.start()
        ghost.stop()
        ghost.delete()
        ghosts.append(ghost_instance)

    # set ghost status for device
    device.ghost_status = 2
    device.ghost_file = ghost_file


class RouterSelectionDialog(object):

    def __init__(self, devices, text, show_path=False):
        self.show_path = show_path
        self.selected_devices = []
        self.devices = devices
        self.w = gtk.Dialog("Router configuration", None,
                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
        self.w.set_border_width(5)

        self.liststore = gtk.ListStore(int, str)
        names = [r.name for r in devices]
        names.sort()
        for n in names:
            self.liststore.append([True, n])
        # create the TreeView using liststore
        treeview = gtk.TreeView(self.liststore)
        scroll = gtk.ScrolledWindow()
        scroll.set_policy(gtk.POLICY_AUTOMATIC,
                          gtk.POLICY_AUTOMATIC)
        scroll.add(treeview)

        cell = gtk.CellRendererToggle()
        cell.set_property('activatable', True)
        cell.connect('toggled', self.toggled, None)
        cell1 = gtk.CellRendererText()

        column = gtk.TreeViewColumn('', cell)
        column1 = gtk.TreeViewColumn('device', cell1, text=1)

        treeview.append_column(column)
        treeview.append_column(column1)

        column.add_attribute(cell, "active", 0)
        column1.set_sort_column_id(1)

        table_layout = gtk.VBox(spacing=10)
        label_head = gtk.Label(text)
        table_layout.pack_start(label_head, False)
        table_layout.pack_start(scroll, True)
        self.all_devices_check = gtk.CheckButton("All devices")
        self.all_devices_check.set_active(True)
        self.all_devices_check.connect("clicked", self.toggle_all,
                                       self.all_devices_check)
        table_layout.pack_start(self.all_devices_check, False)
        if show_path:
            label_path = gtk.Label('Folder: ')
            self.button_path = gtk.FileChooserButton('Select a folder')
            self.button_path.set_width_chars(20)
            self.button_path.set_action (gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
            path_layout = gtk.HBox(spacing=0)
            path_layout.pack_start(label_path, False)
            path_layout.pack_start(self.button_path, False)
            table_layout.pack_start(path_layout, False)

        self.w.vbox.pack_start(table_layout)
        hbutton_box = gtk.HButtonBox()
        button = gtk.Button(stock=gtk.STOCK_CANCEL)
        button.connect("clicked", self.button_clicked, gtk.RESPONSE_CANCEL)
        hbutton_box.add(button)
        button = gtk.Button(stock=gtk.STOCK_OK)
        button.connect("clicked", self.button_clicked, gtk.RESPONSE_OK)
        hbutton_box.add(button)
        self.w.action_area.pack_start(hbutton_box)

        self.w.show_all()

        # display a maximum of 7 rows
        height = treeview.get_background_area(0, column).height
        mheight = min(7, len(self.liststore)+1) * height
        treeview.set_size_request(width=0, height=mheight)

        ret = self.w.run()
        self.w.hide()

    def button_clicked(self, widget, data):
        if data == gtk.RESPONSE_OK:
            for i in range(len(self.liststore)):
                if self.liststore[i][0]:
                    for d in self.devices:
                        if (self.liststore[i][1]==d.name):
                            self.selected_devices.append(d)
        if self.show_path:
            self.path = self.button_path.get_filename()
        self.w.destroy()

    def toggled(self, cell, path, new_text):
        """Callback called when the user toggle one of the checkbox for a
        router."""
        self.liststore[path][0] = not self.liststore[path][0]
        selected = 0
        for i in range(len(self.liststore)):
            if self.liststore[i][0]:
                selected += 1
        if selected == len(self.liststore):
            self.donotswap = True
            self.all_devices_check.set_active(True)
        else:
            self.donotswap = True
            self.all_devices_check.set_active(False)

    def toggle_all(self, widget, data):
        """Callback called when the "select all" checkbox is toggled."""
        if self.donotswap: # calling the method ourself, in toggled method
            self.donotswap = False
            return
        val = widget.get_active()
        for i in range(len(self.liststore)):
            self.liststore[i][0] = val


